'use client'

import { ConnectButton } from '@rainbow-me/rainbowkit'
import { useAccount, useNetwork, useSigner } from 'wagmi'
import { ethConnect, disconnectWeb3 } from '@lit-protocol/lit-node-client'
import { LOCAL_STORAGE_KEYS } from '@lit-protocol/constants'
import { useEffect, useState } from 'react'
import { PKPEthersWallet } from "@lit-protocol/pkp-ethers";


export default function Home() {
  const [authSig, setAuthSig] = useState(null)
  const [error, setError] = useState(null)

  const { address, isConnected, isDisconnected } = useAccount()
  const { data: signer } = useSigner()
  const { chain } = useNetwork()

  async function generateAuthSig(signer, address, chainId) {
    setAuthSig(null);
    setError(null);
    try {
      const newAuthSig = await ethConnect.signAndSaveAuthMessage({
        web3: signer.provider,
        account: address.toLowerCase(),
        chainId: chainId,
      });
      setAuthSig(newAuthSig);
    } catch (err) {
      console.error(err);
      setError(`Failed to sign auth message: ${err.message}`);
    }
  }


  async function sendTXethereum(){
    
    const pkpWallet = new PKPEthersWallet({
      controllerAuthSig: authSig,
      pkpPubKey: "046da3ba67065fd1e2726242ca01cd4601524893f4aa4b0042578fa6cbec28fa8c9a28eb9f7893932fc09717edc9e1db57e157a21eed346247c1db5a722a01f571",
      rpc: "https://rpc.eth.gateway.fm",
    });
    await pkpWallet.init();
    console.log(pkpWallet);

    const from = "0x06B6BE47c86cfcDF3f77c0e17e7aD8af750782aE";
    const to = "0x1A5cfC9EA11afb50011F847fb7dC07bA1e18b05A";
    const value = BigInt(1000000000000000);
    const chainId = 1;
    const gasLimit = 21000;

    // @lit-protocol/pkp-ethers will automatically add missing fields (nonce, chainId, gasPrice, gasLimit)
    const transactionRequest = {
      from,
      to,
      value,
      chainId,
      gasLimit,
    };

    const signedTransactionRequest = await pkpWallet.signTransaction(
      transactionRequest
    );

    await pkpWallet.sendTransaction(signedTransactionRequest);
  }

  async function sendTXgnosis(){
    
    const pkpWallet = new PKPEthersWallet({
      controllerAuthSig: authSig,
      pkpPubKey: "046da3ba67065fd1e2726242ca01cd4601524893f4aa4b0042578fa6cbec28fa8c9a28eb9f7893932fc09717edc9e1db57e157a21eed346247c1db5a722a01f571",
      rpc: "https://rpc.gnosischain.com/",
    });
    await pkpWallet.init();
    console.log(pkpWallet);

    const from = "0x06B6BE47c86cfcDF3f77c0e17e7aD8af750782aE";
    const to = "0x1A5cfC9EA11afb50011F847fb7dC07bA1e18b05A";
    const value = BigInt(1000000000000000);

    // @lit-protocol/pkp-ethers will automatically add missing fields (nonce, chainId, gasPrice, gasLimit)
    const transactionRequest = {
      from,
      to,
      value,
    };

    const signedTransactionRequest = await pkpWallet.signTransaction(
      transactionRequest
    );

    await pkpWallet.sendTransaction(signedTransactionRequest);
  }


  // Fetch auth sig from local storage
  useEffect(() => {
    if (!authSig) {
      const storedAuthSig = localStorage.getItem(LOCAL_STORAGE_KEYS.AUTH_SIGNATURE);
      if (storedAuthSig) {
        const parsedAuthSig = JSON.parse(storedAuthSig);
        setAuthSig(parsedAuthSig);
      }
    }
  }, [authSig])
  
  // Generate auth sig if not already generated and if wallet is connected
  useEffect(() => {
    if (!authSig && isConnected && signer) {
      generateAuthSig(signer, address, chain.id);
    }
  }, [authSig, isConnected, signer, address, chain])

  // Clear auth sig if wallet is disconnected
  useEffect(() => {
    if (isDisconnected) {
      disconnectWeb3();
      setAuthSig(null);
    }
  }, [isDisconnected])

  return (
    <main className="main">
      {error && 
        <div className='alert alert--error'>
          <p>❗️ {error}</p>
          <button className='alert__btn' onClick={() => generateAuthSig(signer, address, chain.id)}>
            Try again
          </button>
        </div>
      }
      {authSig && 
        <p className='alert alert--success'>🔐 Auth sig has been generated and stored in local storage under {' '}
          <code>{LOCAL_STORAGE_KEYS.AUTH_SIGNATURE}</code>
          !
        </p>
      }
      <ConnectButton />
      <button style={{marginTop: "2rem", padding: "1rem"}} onClick={() => sendTXgnosis()}>
        send 0.001 xdai
      </button>

      <button style={{marginTop: "2rem", padding: "1rem"}} onClick={() => sendTXethereum()}>
        send 0.001 eth
      </button>
   
    </main>
  )
}
